;********************************************************************************
;**									       **
;*				LOGIC MODULES for PCEM				*
;**									       **
;********************************************************************************



;--- COUNTER MODULE -------------------------------------------------------------
MDL_CTR	macro
;--- COUNTER CLOCK ----------------
	btfsc	CLK		;clk=1 ?
	btfsc	NOT_R_W		;-R/W=0 ?
	goto	C_NO_CLOCK	;no, clock acive in L	
	btfsc	CIP		;yes, counter in progress?
	goto	C_FINAL		;yes, inc(BP) already done...
	incf	BP, same	;no, inc(BP)
	bsf	CIP		;BP incremented, set counter in progress
	goto	C_FINAL		;finalize...
C_NO_CLOCK	
	bcf	CIP		;counter isn't in progress
C_FINAL
;--- COUNTER RESET ----
	btfsc	CLK		;clk=1 ?
	btfsc	NOT_RESET	;yes, rst=0?
	goto	_SKIP
	clrf	BP		;yes...	
_SKIP
	endm

;--- MEMORY MODULE --------------------------------------------------------------
MDL_MEM	macro

;------------ GET ADDRESS ----------------
;send bit addressed by counter (BP)	

	movf	BP, W

;-- GET BYTE ADDRESS FROM BIT ADDRESS
;-----------------------------------------
;GET_ADDR
;BFP must be set in W (for bit address modification)

	movwf	ACC1		;whole adress: ACC1
	andlw	b'00000111'	;bit ptr. required
	movwf	BPOS		;store bit-offset

	rrf	ACC1, same	;3st bits not required (point to bits)
	rrf	ACC1, same
	rrf	ACC1, W		;now, byte address in W
	andlw	b'00011111'	;FILTER against MAD data

	movwf	ACC2		;store...

;--------------------------
	movlw	SAUF		;start address of units field...
	subwf	BP, W		;read only sector or read/write sec.?
	btfss	STATUS, C	;C=0->RO(descriptor), C=1->RW (units)
	goto	MEM_RO_DSR	;read only sector...

;************************ READ/ WRITE sector ************************* 60-FF

;--- READ ---------------------------------------

;READ/WRITE?

	btfsc	CLK		;memory enabled for output?
	goto	MEM_WRITE	;no, write seq.


;read/write sector:
;moved GET_UBYTE,	

	movf	ACC2, W		;byte addresed by ACC2, data to ACC1
	movwf	FSR		;set address for IDA

	movf	INDF, W		;data ready in W
	movwf	ACC1

;get bit....

	movf	BPOS, W		;store shift length..	

	btfsc	STATUS, Z	;BPTR = 0?
	goto	BIT_GOT1	;Yes, bit ready to send

	bcf	STATUS, C	;no more '1's

GET_BIT_LOOP1
	rrf	ACC1, same	;shift bit - next bit addr.
	decfsz	BPOS, same
	goto	GET_BIT_LOOP1

;value in ACC2 preset by GET_BIT_ADDR	

BIT_GOT1
	movwf	BPOS		;refresh shift length...

;end, addressed bit is stored in ACC as 1st bit

	goto	MEM_SBIT	;send bit on port OUT

;--- WRITE --------------------------------------

MEM_WRITE
;write?	
	btfss	NOT_R_W		;yes, -R/W=1?
	goto	MEM_DONE	;no...
;read data...
;call Get_UBYTE...

	movf	ACC2, W		;byte addresed by ACC2, data to ACC1
	movwf	FSR		;set address for IDA

	movf	INDF, W		;data ready in W
	movwf	ACC1

;GET_BIT...

	movf	BPOS, W		;store shift length..	

	btfsc	STATUS, Z	;BPTR = 0?
	goto	BIT_GOT2	;Yes, bit ready to send

	bcf	STATUS, C	;no more '1's

GET_BIT_LOOP2

	rrf	ACC1, same	;shift bit - next bit addr.
	decfsz	BPOS, same
	goto	GET_BIT_LOOP2

;value in ACC2 preset by GET_BIT_ADDR

BIT_GOT2
	movwf	BPOS		;refresh shift length...
;end, addressed bit is stored in ACC as 1st bit


;modify data...
	bsf	ACC1, 0		;set BIT...

;write data...
;moved proc. set_bit...
	
	movf	BPOS, W		;store shift length (0-7)
	
	btfsc	STATUS, Z	;BPTR = 0?
	goto	BIT_SET_DONE1	;Yes, bit is on right location

BIT_SHIFT_LOOP1
	rlf	ACC1, same	;shift bit - next bit addr.
	decfsz	BPOS, same	
	goto	BIT_SHIFT_LOOP1

BIT_SET_DONE1
	movwf	BPOS		;refresh shift length...
	
;store data...

	movf	ACC2, W
	movwf	FSR		;set address for IDA

	movf	ACC1, W
	movwf	INDF		;data ready in W

;write seq. done...
	goto	MEM_DONE	;done
		

;************************ READ ONLY sector ************************* 0-5F
MEM_RO_DSR

;READ/WRITE?
	btfsc	CLK		;memory enabled for output?
	goto	MEM_DONE	;no, output disabled

	movf	ACC2, W
	call	CARD_DSR	;get descriptor's bits...
	movwf	ACC1		;store...

;GET_BIT
	movf	BPOS, W		;store shift length..	

	btfsc	STATUS, Z	;BPTR = 0?
	goto	BIT_GOT3		;Yes, bit ready to send

	bcf	STATUS, C	;no more '1's

GET_BIT_LOOP3
	rrf	ACC1, same	;shift bit - next bit addr.
	decfsz	BPOS, same
	goto	GET_BIT_LOOP3

;value in ACC2 preset by GET_BIT_ADDR

BIT_GOT3
	movwf	BPOS		;refresh shift length...

;end, addressed bit is stored in ACC as 1st bit


;send bit:
MEM_SBIT	
	btfss	ACC1, 0		;LOW?
	bcf	OUT		;yes
	btfsc	ACC1, 0		;no, HIGH?
	bsf	OUT	
MEM_DONE
	endm



