;********************************************************************************
;**									       **
;*				LOGIC MODULES for PCEM				*
;**									       **
;********************************************************************************



;--- COUNTER MODULE -------------------------------------------------------------
MDL_CTR
;--- COUNTER CLOCK ----------------
	btfsc	CLK		;clk=1 ?
	btfsc	NOT_R_W		;-R/W=0 ?
	goto	C_NO_CLOCK	;no, clock acive in L	
	btfsc	CIP		;yes, counter in progress?
	goto	C_FINAL		;yes, inc(BP) already done...
	incf	BP, same	;no, inc(BP)
	bsf	CIP		;BP incremented, set counter in progress
C_FINAL
;--- COUNTER RESET ----
	btfsc	CLK		;clk=1 ?
	btfsc	NOT_RESET	;yes, rst=0?
	retlw	0FFh
	clrf	BP		;yes...
	retlw	00h
C_NO_CLOCK	
	bcf	CIP		;counter isn't in progress
	goto	C_FINAL		;finalize...


;--- MEMORY MODULE --------------------------------------------------------------
MDL_MEM

;------------ GET ADDRESS ----------------
;send bit addressed by counter (BP)	

	movf	BP, W

;-- GET BYTE ADDRESS FROM BIT ADDRESS
;-----------------------------------------
;GET_ADDR
;BFP must be set in W (for bit address modification)

	movwf	ACC1		;whole adress: ACC1
	andlw	b'00000111'	;bit ptr. required
	movwf	BPOS		;store bit-offset

	rrf	ACC1, same	;3st bits not required (point to bits)
	rrf	ACC1, same
	rrf	ACC1, W		;now, byte address in W
	andlw	b'00011111'	;FILTER against MAD data

	movwf	ACC2		;store...

;--------------------------
	movlw	SAUF		;start address of units field...
	subwf	BP, W		;read only sector or read/write sec.?
	btfss	STATUS, C	;C=0->RO(descriptor), C=1->RW (units)
	goto	MEM_RO_DSR	;read only sector...

;************************ READ/ WRITE sector ************************* 60-FF

;--- READ ---------------------------------------

;READ/WRITE?

	btfsc	CLK		;memory enabled for output?
	goto	MEM_WRITE	;no, write seq.


;read/write sector:
	call	GET_UBYTE	;byte addresed by ACC2, data to ACC1
	call	GET_BIT		;bit shift R
	goto	MEM_SBIT	;send bit on port OUT

;--- WRITE --------------------------------------

MEM_WRITE
;write?	
	btfss	NOT_R_W		;yes, -R/W=1?
	retlw	0FFh		;no...
;read data...
	call	GET_UBYTE	;byte addresed by ACC2, data to ACC1
	call	GET_BIT		;bit shift R

;modify data...
	bsf	ACC1, 0		;set BIT...

;write data...
	call	SET_BIT		;bit shift L
	call	SET_UBYTE	;store data
;write seq. done...
	retlw	00h		;OP done
		

;************************ READ ONLY sector ************************* 0-5F
MEM_RO_DSR

;READ/WRITE?
	btfsc	CLK		;memory enabled for output?
	retlw	0FFh		;no, output disabled

	movf	ACC2, W
	call	CARD_DSR	;get descriptor's bits...
	movwf	ACC1		;store...
	call	GET_BIT		;bit shift

;send bit:
MEM_SBIT	
	btfss	ACC1, 0		;LOW?
	bcf	OUT		;yes
	btfsc	ACC1, 0		;no, HIGH?
	bsf	OUT
	retlw	00h


;--------------------------------------------------------------------------------
;-- GET BIT FROM ACC1, bit0 --				DATA: ACC1->BPOS[ACC1[0]]
;--------------------------------------------------------------------------------

GET_BIT
	movf	BPOS, W		;store shift length..	

	btfsc	STATUS, Z	;BPTR = 0?
	goto	BIT_GOT		;Yes, bit ready to send

	bcf	STATUS, C	;no more '1's

GET_BIT_LOOP
	rrf	ACC1, same	;shift bit - next bit addr.
	decfsz	BPOS, same
	goto	GET_BIT_LOOP

;value in ACC2 preset by GET_BIT_ADDR

	

BIT_GOT
	movwf	BPOS		;refresh shift length...
	retlw	0FFh		;end, addressed bit is stored in ACC as 1st bit

;--------------------------------------------------------------------------------
;-- SET BIT IN ACC1 (modified bit put to right location) --	DATA: ACC1->ACC1
;--------------------------------------------------------------------------------

SET_BIT
	movf	BPOS, W		;store shift length (0-7)
	
	btfsc	STATUS, Z	;BPTR = 0?
	goto	BIT_SET_DONE	;Yes, bit is on right location

BIT_SHIFT_LOOP
	rlf	ACC1, same	;shift bit - next bit addr.
	decfsz	BPOS, same	
	goto	BIT_SHIFT_LOOP

BIT_SET_DONE
	movwf	BPOS		;refresh shift length...
	retlw	00h



;--------------------------------------------------------------------------------
;-- GET UNIT's BIT FIELD BYTE --				DATA @ACC2->ACC1
;--------------------------------------------------------------------------------
GET_UBYTE
	movf	ACC2, W
	movwf	FSR		;set address for IDA

	movf	INDF, W		;data ready in W
	movwf	ACC1

	retlw	0FFh

;--------------------------------------------------------------------------------
;-- SET UNIT's BIT FIELD BYTE --				DATA ACC1->@ACC2
;--------------------------------------------------------------------------------
SET_UBYTE
	movf	ACC2, W
	movwf	FSR		;set address for IDA

	movf	ACC1, W
	movwf	INDF		;data ready in W

	retlw	0FFh

;--------------------------------------------------------------------------------
