;********************************************************************************
;**									       **
;*				Phone Card Emulator				*
;**									       **
;********************************************************************************

;-- Processor ---

	LIST	p=12c508
;not required here	__config	b'000000001110'

	#include "p12C508.inc"
	#include "pcem_dsr.asm"

;-- destination -----------------------
W		equ		0
same		equ		1


;-- constants -------------------------
INI_OPTION	equ		b'10011010'
INI_TRIS	equ		b'11111101'
INI_GPIO	equ		b'00000000'
UFSA		equ		D'96'		;"Units Field" Start Address
RAMADB		equ		0x14		;RAM Address of Describe Bits (as BFs)
LDSRB		equ		D'95'		;last describe bit address
DSR_BYTES	equ		D'12'
;--------------------------------------
OSC_CALIB	equ		080h		;internal RC osc. calibration

;-- registers -------------------------
;IDA		equ		0x00	;Indirect data Addressing
BFP		equ		0x07	;Bit Array Pointer
EUT		equ		0x08	;# of Erased UniTs
ACC1		equ		0x09
ACC2		equ		0x0A
BPOS		equ		0x0B	;bit-position in byte (used by GET_BIT_ADDR)
BFs		equ		0x14	;start of bit field

;-- bits ------------------------------
#define		Vpp		GPIO, 0	;EPROM programming voltage
#define		DATA_OUT	GPIO, 1	;data out
#define		CHECK_Vpp	GPIO, 2	;enable checking Vpp in emulated programming EPROM
#define		NOT_RESET	GPIO, 3	;Card -RESET
#define		CLK		GPIO, 4	;clock signal ~25Hz
#define		NOT_R_W		GPIO, 5	;Read / Write data

;-- address' --------------------------
Subrout		equ		0x001
Start		equ		0x100
ResetVector	equ		0x000
OscCalib	equ		0x1FF

;********************************************************************************
;*				MAIN PROGRAM					*
;********************************************************************************

	org	OscCalib

	movlw	OSC_CALIB

;-------------------------------

	org	ResetVector
	goto	Start

;-------------------------------
	org	Start

;oscillator calibration...

	movwf	OSCCAL

;initialize option

	movlw	INI_OPTION	;initialize OPTION
	option

;power on ?
	
	btfss	STATUS, NOT_TO
	goto	NO_PWRUP	;No

;----------------
	clrwdt
;----------------

	clrf	BFP		;bit field pointer null
	clrf	EUT		;used units=0
	call	LOAD_DSR_FIELD	;load, ID, serial#, #units...(describe field)
	

NO_PWRUP
;initialize tris, gpio
	
	movlw	INI_GPIO	;initialize GPIO
	movwf	GPIO

	movlw	INI_TRIS	;initialize TRIS
	tris	GPIO

;CLK = Hi?

MAIN_LOOP

;----------------
	clrwdt
;----------------

SCAN_CLK_HI
	btfss	CLK		;new sync. signal?
	goto	SCAN_CLK_HI

;----------------
	clrwdt
;----------------

;	  |---	
;CLK = ---|
	
;RESET required ?

	btfss	NOT_RESET	
	call	CARD_RESET	;Yes

;Read/Write?

	btfsc	NOT_R_W
	goto	WRITE_BIT	;Yes, write to itself

;Send BIT...
	
	movlw	UFSA		;Units Field Start Address
	subwf	BFP, W
	btfsc	STATUS, C	;BFP<60? (UFSA) (C=0)
	goto	SEND_UNITSF	;No, Send Units Field...

;IDs, serial# ...DESCRIBE BITS....
	
	call	GET_DSR_BIT	;get describe bits

;SEND BIT ON PORT
;this instr. block used for no noise on bus...

	btfsc	ACC1, 0		;1st (0) bit = 1?
	bsf	DATA_OUT	;Yes, set log1
	btfss	ACC1, 0		;1st (0) bit = 0?
	bcf	DATA_OUT

;bit ready on DATA_OUT

	goto	SCAN_CLK_LO
;---------------------------- MEMORY SKIPPED BY BIT SEND --------------------	
;send emulated units field

SEND_UNITSF
	movlw	LDSRB		;last describe bit address
	addwf	EUT, W
	movwf	ACC1
	movf	BFP, W
	subwf	ACC1, W		;BFP>EUT+5Fh?

;this instr. block used for no noise on bus...

	btfsc	STATUS, C
	bsf	DATA_OUT	;No, send> "used unit"
	btfss	STATUS, C
	bcf	DATA_OUT	;Yes, send> not used unit

;used/not used units transmittion done...

	goto	SCAN_CLK_LO

;---------------------------- MEMORY SKIPPED BY SEND_UNITS_(F)IELD --------------------

;"write" to emulated EPROM
WRITE_BIT
	btfss	CHECK_Vpp	;Vpp checking enabled?
	goto	NO_Vpp_SCAN	;skip next instr.

	btfss	Vpp		;Vpp at 1?
	goto	SCAN_CLK_LO	;No, exit

NO_Vpp_SCAN
	movlw	UFSA		;unit Field Start Address
	subwf	BFP, W		;BFP<60?
	btfss	STATUS, C
	goto	MODIFY_DSR	;Modify Describe bits

	incf	EUT, same		;next used unit
	goto	SCAN_CLK_LO

;Modify Decribe bits

MODIFY_DSR
	call	GET_DSR_BIT	;stored in ACC1
	bsf	ACC1, 0		;modify bit

;!!! you musn't effect C !!!
;shift bit to right position...(n x rlf)

	movf	BPOS, W		;set shift length (0-7)
	movwf	ACC2

BIT_SHIFT_LOOP2
	movf	ACC2, same	;effect STATUS
	btfsc	STATUS, Z	;BPTR = 0?
	goto	BIT_SHIFTED2	;Yes, bit ready to send
	rlf	ACC1, same	;shift bit - next bit addr.
	decf	ACC2, same	
	goto	BIT_SHIFT_LOOP2
BIT_SHIFTED2

;save modified data....
	movf	ACC1, W
	movwf	INDF		;save data
;op done...

;------------------------------------------------------------------------------	
	
;WAIT for CLK=0

;----------------
	clrwdt
;----------------

SCAN_CLK_LO
	btfsc	CLK		;CLK=0?
	goto	SCAN_CLK_LO

;----------------
	clrwdt
;----------------

;-RESET active?

	btfsc	NOT_RESET	;-RESET =0?
	incf	BFP, same	;No, point to next bit
	goto	MAIN_LOOP
	

;********************************************************************************
;*				SUBROUTINES					*
;********************************************************************************

	org	Subrout

;--------------------------------------------------------------------------------
;-- CARD DESCRIPTOR ---
;--------------------------------------------------------------------------------
CARD_DSR
	andlw	b'00001111'	;FILTER against MAD data
	addwf	PCL, same
	retlw	00h		;reserved for decfsz instr. (see LOAD_DSR_FIELD\LOAD_LOOP)
	retlw	BF0_7		;byte 0
	retlw	BF8_15		;byte 1
	retlw	BF16_23		;byte 2
	retlw	BF24_31		;byte 3
	retlw	BF32_39		;byte 4
	retlw	BF40_47		;byte 5
	retlw	BF48_55		;byte 6
	retlw	BF56_63		;byte 7
	retlw	BF64_71		;byte 8
	retlw	BF72_79		;byte 9
	retlw	BF80_87		;byte 10
	retlw	BF88_95		;byte 11
	retlw	00h		;reserved for MAD data
	retlw	00h
	retlw	00h



;--------------------------------------------------------------------------------
;-- LOAD DESCRIBE BITS FIELD --
;--------------------------------------------------------------------------------
LOAD_DSR_FIELD
	movlw	DSR_BYTES	;# of DSR bytes
	movwf	ACC1
	movlw	01Fh		;set FSR=FFh / describe bits field end address
	movwf	FSR
LOAD_LOOP
	movf	ACC1, W
	call	CARD_DSR
	movwf	INDF		;save data to RAM
	decf	FSR, same	;point to "next" byte
	decfsz	ACC1, same	;all done?
	goto	LOAD_LOOP	;No
	
	retlw	00h		;Yes



;--------------------------------------------------------------------------------
;-- GET BIT ADDRESSED BY BFP --
;--------------------------------------------------------------------------------
GET_DSR_BIT

	call	GET_BIT_ADDR
	movf	INDF, W		;save data  for bit shift
	movwf	ACC1

BIT_SHIFT_LOOP
	movf	ACC2, same	;effect STATUS
	btfsc	STATUS, Z	;BPTR = 0?
	goto	BIT_SHIFTED	;Yes, bit ready to send
	rrf	ACC1, same	;shift bit - next bit addr.
	decf	ACC2, same

;value in ACC2 preset by GET_BIT_ADDR

	goto	BIT_SHIFT_LOOP

BIT_SHIFTED
	retlw	0FFh		;end, addressed bit is stored in ACC as 1st bit

;--------------------------------------------------------------------------------
;-- GET BYTE ADDRESS FROM BIT ADDRESS --
;--------------------------------------------------------------------------------
GET_BIT_ADDR
	movf	BFP, W
	andlw	b'00000111'	;bit ptr. required
	movwf	BPOS		;Bit-position
	movwf	ACC2

	rrf	BFP, W		;3st bits not required (point to bits)
	movwf	ACC1
	rrf	ACC1, same
	rrf	ACC1, W		;now, byte address in W
	andlw	b'00001111'	;FILTER against MAD data
;
	movwf	FSR		;set IDA
	movlw	RAMADB		;RAM Address of Describe Bits
	addwf	FSR, same	;shift address
;
	btfsc	STATUS, C	;address OK, not overflowed?
ADDR_ERR
	goto	ADDR_ERR	;No, restart PICmicro

	retlw	0FFh


;--------------------------------------------------------------------------------
;-- CARD RESET --
;--------------------------------------------------------------------------------

CARD_RESET
	clrf	BFP		;bit pointer null

;----------------
;	clrwdt
;----------------

TEST_CLK_RST
	btfsc	CLK		;wait for CLK=0
	goto	TEST_CLK_RST
	retlw	0FFh


	end
