;********************************************************************************
;**									       **
;*				Phone Card Emulator				*
;**			>software emulation NMOS technology<		       **
;********************************************************************************

;-- Processor ---

	LIST	p=12c508
;not required here	__config	b'000000001110'

	#include "p12C508.inc"
	#include "card_dsr.asm"	
	#include "modules3.asm"

;-- destination -----------------------
W		equ		0
same		equ		1


;-- constants -------------------------
INI_OPTION	equ		b'10011000'
_TRIS_0		equ		b'11111110'	;NMOS - emulation (log0, OUT tied to GND)
_TRIS_1		equ		b'11111111'	;NMOS - emulation (log1, OUT pulluped to Vcc)
INI_TRIS 	equ		b'11111111'
INI_GPIO	equ		b'00000000'	;switching status.. (NMOS)
SYS_DEFAULT	equ		b'10000000'
UFRAM		equ		D'12'		;Units Field RAM address
UNITS_START	equ		0x0C		;RAM - start of unit's field

RAM_START	equ		0x07		;user RAM start address
SARE		equ		0FFh		;sample rate (wait loop) - inverted value
SAUF		equ		060h		;start address of units field
SKOV		equ		0FFh		;skip old version (active in 00h)
FACTORY_DEF	equ		b'00000011'	;delete 2 units..
;--------------------------------------
OSC_CALIB	equ		070h		;internal RC osc. calibration

;-- registers -------------------------
BP		equ		0x07	;Bit Array Pointer
SYS		equ		0x08	;system. reg.
ACC1		equ		0x09	;uni. ACCumulator
ACC2		equ		0x0A	;uni. ACCumulator
BPOS		equ		0x0B	;bit-position in byte (used by GET_BIT_ADDR)
BFs		equ		0x14	;start of bit field

;-- bits ------------------------------


#define		OUT		GPIO, 0	;data out
#define		NOT_RESET	GPIO, 4	;Card -RESET
#define		CLK		GPIO, 2	;clock signal ~100Hz
#define		NOT_R_W		GPIO, 5	;Read / Write data

;#define	Vpp		GPIO, X	;EPROM programming voltage
;#define	CHECK_Vpp	GPIO, X	;enable checking Vpp in emulated programming EPROM

;system bits in SYS reg.
;#define		MWO		SYS, 0	;serial mem. write req.
;#define		MEN		SYS, 1	;mem. enable output
;#define		CRST		SYS, 2	;counter reset
;#define		CCLK		SYS, 3	;counter clock

#define		CIP		SYS, 7	;counter in progress flag

;-- address' --------------------------
Subrout		equ		0x001
Start		equ		0x100
ResetVector	equ		0x000
OscCalib	equ		0x1FF

;********************************************************************************
;*				MAIN PROGRAM					*
;********************************************************************************

	org	OscCalib

	movlw	OSC_CALIB

;-------------------------------

	org	ResetVector
	goto	Start

;-------------------------------
	org	Start

;oscillator calibration...

	movwf	OSCCAL

;-------------------------------
;choose user program structure... (defaut/ user...)
	
;	org	CH_USER_BLOCK
	movlw	SKOV			;default: FFh->01h, user: 00h
	andlw	0FFh			;effect Z
	btfsc	STATUS, Z		;new? -upgraded version?
	goto	ADST		;go to NEW PROG. structure...

;initialize option

	movlw	INI_OPTION	;initialize OPTION
	option

;power on ?
	
	btfsc	STATUS, NOT_TO	;power up?
	call	CARD_POR	;yes, power-on card reset...

;----------------
	clrwdt
;----------------

;initialize tris, gpio
	
	movlw	INI_GPIO	;initialize GPIO
	movwf	GPIO

	movlw	INI_TRIS	;initialize TRIS
	tris	GPIO

	movlw	FACTORY_DEF	;delete 2 units...
	movwf	UNITS_START

;--------------------------------------------------------------------------------
;--			Asynchronous features simulation		       --
;--------------------------------------------------------------------------------


LOOP
;----------------
	clrwdt
;----------------
	
;module counter:
	MDL_CTR

;module serial memory:
	MDL_MEM
;loop
	goto	LOOP

;for advanced versions...
;	nop

;******************************** NEW *****************************************
;-------------------	

ADST				;advanced structure

;********************************************************************************
;*				SUBROUTINES					*
;********************************************************************************

	org	Subrout	

;--------------------------------------------------------------------------------
;-- CARD POWER ON RESET ---
;--------------------------------------------------------------------------------

CARD_POR
	clrwdt
	movlw	RAM_START	;start RAM addr.
	movwf	FSR
CLEAR_LOOP
	clrf	INDF
	incfsz	FSR, same
	goto	CLEAR_LOOP
	retlw	00h

;--------------------------------------------------------------------------------
;-- CARD DESCRIPTOR ---
;--------------------------------------------------------------------------------
CARD_DSR
	andlw	b'00001111'	;FILTER against MAD data
	addwf	PCL, same	
	retlw	bs0_7		;byte 0
	retlw	bs8_15		;byte 1
	retlw	bs16_23		;byte 2
	retlw	bs24_31		;byte 3
	retlw	bs32_39		;byte 4
	retlw	bs40_47		;byte 5
	retlw	bs48_55		;byte 6
	retlw	bs56_63		;byte 7
	retlw	bs64_71		;byte 8
	retlw	bs72_79		;byte 9
	retlw	bs80_87		;byte 10
	retlw	bs88_95		;byte 11
	retlw	00h		;reserved for MAD data....
	retlw	00h		;reserved for MAD data....
	retlw	00h
	retlw	00h




;********************************NEW*******************************************

	end
