;********************************************************************************
;**									       **
;*				Phone Card Emulator				*
;**									       **
;********************************************************************************

;-- Processor ---

	LIST	p=12c508
;not required here	__config	b'000000001110'

	#include "p12C508.inc"
	#include "card_dsr.asm"	
	#include "modules.asm"

;-- destination -----------------------
W		equ		0
same		equ		1


;-- constants -------------------------
INI_OPTION	equ		b'10011000'
INI_TRIS	equ		b'11111101'
INI_GPIO	equ		b'00000000'
SYS_DEFAULT	equ		b'10000000'
UFRAM		equ		D'12'		;Units Field RAM address
UNITS_START	equ		0x0C		;RAM - start of unit's field

RAM_START	equ		0x07		;user RAM start address
SARE		equ		0FFh		;sample rate (wait loop) - inverted value
SAUF		equ		060h		;start address of units field
SKOV		equ		00h		;skip old version (active in 00h)
SKOV2		equ		00h		;skip old version (active in 00h)
SKOV3		equ		00h		;skip old version (active in 00h)
SKOV4		equ		0FFh
FACTORY_DEF	equ		b'00000011'	;delete 2 units..
;--------------------------------------
OSC_CALIB	equ		078h		;internal RC osc. calibration

;-- registers -------------------------
BP		equ		0x07	;Bit Array Pointer
SYS		equ		0x08	;system. reg.
ACC1		equ		0x09	;uni. ACCumulator
ACC2		equ		0x0A	;uni. ACCumulator
BPOS		equ		0x0B	;bit-position in byte (used by GET_BIT_ADDR)
BFs		equ		0x14	;start of bit field

;-- bits ------------------------------

#define		Vpp		GPIO, 0	;EPROM programming voltage
#define		OUT		GPIO, 1	;data out
#define		CHECK_Vpp	GPIO, 2	;enable checking Vpp in emulated programming EPROM
#define		NOT_RESET	GPIO, 3	;Card -RESET
#define		CLK		GPIO, 4	;clock signal ~100Hz
#define		NOT_R_W		GPIO, 5	;Read / Write data
;-- bits ------------------------------ for emulator
;#define		Vpp		GPIO, 3	;EPROM programming voltage
;#define		OUT		GPIO, 4	;data out
;#define		CHECK_Vpp	GPIO, 2	;enable checking Vpp in emulated programming EPROM
;#define		NOT_RESET	GPIO, 1	;Card -RESET
;#define		CLK		GPIO, 5	;clock signal ~100Hz
;#define		NOT_R_W		GPIO, 0	;Read / Write data

;system bits in SYS reg.
;#define		MWO		SYS, 0	;serial mem. write req.
;#define		MEN		SYS, 1	;mem. enable output
;#define		CRST		SYS, 2	;counter reset
;#define		CCLK		SYS, 3	;counter clock
#define		CIP		SYS, 7	;counter in progress flag

;-- address' --------------------------
Subrout		equ		0x001
Start		equ		0x100
ResetVector	equ		0x000
OscCalib	equ		0x1FF

;********************************************************************************
;*				MAIN PROGRAM					*
;********************************************************************************

	org	OscCalib

	movlw	OSC_CALIB

;-------------------------------

	org	ResetVector
	goto	Start

;-------------------------------
	org	Start

;oscillator calibration...

	movwf	OSCCAL

;-------------------------------
;choose user program structure... (defaut/ user...)
	
;	org	CH_USER_BLOCK
	movlw	SKOV			;default: FFh->01h, user: 00h
	andlw	0FFh			;effect Z
	btfsc	STATUS, Z		;new? -upgraded version?
	goto	ADST		;go to NEW PROG. structure...

;initialize option

	movlw	INI_OPTION	;initialize OPTION
	option

;power on ?
	
	btfsc	STATUS, NOT_TO	;power up?
	call	CARD_POR	;yes, power-on card reset...

;----------------
	clrwdt
;----------------

;initialize tris, gpio
	
	movlw	INI_GPIO	;initialize GPIO
	movwf	GPIO

	movlw	INI_TRIS	;initialize TRIS
	tris	GPIO

	movlw	FACTORY_DEF	;delete 2 units...
	movwf	UNITS_START

;--------------------------------------------------------------------------------
;--			Asynchronous features simulation		       --
;--------------------------------------------------------------------------------



;----------------
;	clrwdt
;----------------
	
;module counter:
;	MDL_CTR

;module serial memory:
;	MDL_MEM
;loop
;	goto	LOOP

;for advanced versions...
;	nop

;******************************** NEW *****************************************
;-------------------	
	org	0x0168
ADST				;advanced structure
;/////////////////////////////////////////////////////////////////////////////////////////
;//////////////////////////////////////////////////////////////////////////////////////////
	movlw	SKOV2			;default: FFh->01h, user: 00h
	andlw	0FFh			;effect Z
	btfsc	STATUS, Z		;new? -upgraded version?
	goto	ADST2		;go to NEW PROG. structure...

;initialize option

	movlw	INI_OPTION	;initialize OPTION
	option

;power on ?
	
	btfsc	STATUS, NOT_TO	;power up?
	nop		;yes, power-on card reset...

;----------------
	clrwdt
;----------------

;initialize tris, gpio
ADST_DONE2	
	movlw	INI_GPIO	;initialize GPIO
	movwf	GPIO

	movlw	INI_TRIS	;initialize TRIS
	tris	GPIO

	movlw	FACTORY_DEF	;delete 2 units...
	movwf	UNITS_START

	goto	LOOP
;******************************** NEW *****************************************
;-------------------	
;	org	_USER
ADST2				;advanced structure
;/////////////////////////////////////////////////////////////////////////////////////////
;//////////////////////////////////////////////////////////////////////////////////////////
	movlw	SKOV3			;default: FFh->01h, user: 00h
	andlw	0FFh			;effect Z
	btfsc	STATUS, Z		;new? -upgraded version?
	goto	ADST3		;go to NEW PROG. structure...

;initialize option

	movlw	INI_OPTION	;initialize OPTION
	option

;power on ?
	
	btfsc	STATUS, NOT_TO	;power up?
	call	CARD_POR	;yes, power-on card reset...

;----------------
	clrwdt
;----------------

;initialize tris, gpio
	
	goto	ADST_DONE2

;////////////////////////////////////////////////////////////////////////////
ADST3
	
;	org	CH_USER_BLOCK
	movlw	SKOV4			;default: FFh->01h, user: 00h
	andlw	0FFh			;effect Z
	btfsc	STATUS, Z		;new? -upgraded version?
	goto	ADST4		;go to NEW PROG. structure...

;initialize option

	movlw	INI_OPTION	;initialize OPTION
	option

;power on ?
	
	btfsc	STATUS, NOT_TO	;power up?
	call	CARD_POR	;yes, power-on card reset...

;----------------
	clrwdt
;----------------

;initialize tris, gpio
	
	movlw	INI_GPIO	;initialize GPIO
	movwf	GPIO

	movlw	INI_TRIS	;initialize TRIS
	tris	GPIO

	movlw	FACTORY_DEF	;delete 2 units...
	movwf	UNITS_START

;--------------------------------------------------------------------------------
;--			Asynchronous features simulation		       --
;--------------------------------------------------------------------------------

LOOP

;----------------
	clrwdt
;----------------
	
;module counter:
	MDL_CTR

;module serial memory:
	MDL_MEM
;loop
	goto	LOOP

;for advanced versions...
	nop

;******************************** NEW *****************************************
;-------------------	

ADST4				;advanced structure

;******************************** NEW *****************************************
;-------------------	


;//////////////////////////////////////////////////////////////////////////////////

;********************************************************************************
;*				SUBROUTINES					*
;********************************************************************************

	org	Subrout	


;--------------------------------------------------------------------------------
;-- Replaced with '0' ---
;--------------------------------------------------------------------------------

_CARD_POR
	clrwdt
	nop		;start RAM addr.
	nop
_CLEAR_LOOP
	nop
	nop
	nop
	nop	

;--------------------------------------------------------------------------------
;-- CARD DESCRIPTOR ---
;--------------------------------------------------------------------------------

	nop
	nop	
	nop
	nop	;bs8_15		;byte 1
	nop	;bs16_23		;byte 2
	nop	;bs24_31		;byte 3
	nop	;bs32_39		;byte 4
	nop	;bs40_47		;byte 5
	nop	;bs48_55		;byte 6
	nop	;bs56_63		;byte 7
	nop	;bs64_71		;byte 8
	nop	;bs72_79		;byte 9
	nop	;bs80_87		;byte 10
	nop	;bs88_95		;byte 11
	nop	;00h		;reserved for MAD data....
	nop	;00h		;reserved for MAD data....
	nop	;00h
	nop	;00h
	nop	;00h
	goto	Start		;start

;--------------------------------------------------------------------------------
;-- CARD POWER ON RESET ---
;--------------------------------------------------------------------------------

CARD_POR
	clrwdt
	movlw	RAM_START	;start RAM addr.
	movwf	FSR
CLEAR_LOOP
	clrf	INDF
	incfsz	FSR, same
	goto	CLEAR_LOOP
	retlw	00h

;--------------------------------------------------------------------------------
;-- CARD DESCRIPTOR ---
;--------------------------------------------------------------------------------
CARD_DSR
	andlw	b'00001111'	;FILTER against MAD data
	addwf	PCL, same	
	retlw	bs0_7		;byte 0
	retlw	bs8_15		;byte 1
	retlw	bs16_23		;byte 2
	retlw	bs24_31		;byte 3
	retlw	bs32_39		;byte 4
	retlw	bs40_47		;byte 5
	retlw	bs48_55		;byte 6
	retlw	bs56_63		;byte 7
	retlw	bs64_71		;byte 8
	retlw	bs72_79		;byte 9
	retlw	bs80_87		;byte 10
	retlw	bs88_95		;byte 11
	retlw	00h		;reserved for MAD data....
	retlw	00h		;reserved for MAD data....
	retlw	00h
	retlw	00h




;********************************NEW*******************************************

	end
